package cnf;

import static org.junit.Assert.*;

import org.junit.Test;
import cnf.*;

public class _test_Formula {

	@Test
	public void test_empty_formula() {
		Formula f = new Formula();
		Environment e = new Environment();
		assertEquals(f.eval(e), Bool.UNDEFINED);
	}
	
	@Test
	public void test_one_empty_clause(){
		Formula f = new Formula();
		Clause c = new Clause();
		Environment e = new Environment();
		f.add(c);
		assertEquals(f.eval(e), Bool.UNDEFINED);
	}
	
	@Test
	public void test_one_clause(){
		Formula f = new Formula();
		Clause c = new Clause();
		Literal l = new Literal(0);
		Environment e = new Environment();
		e.add(0, Bool.TRUE);
		f.add(c);
		c.add(l);
		assertEquals(f.eval(e), Bool.TRUE);
	}	
	
	@Test
	public void test_1000_true_clauses(){
		Formula f = new Formula();
		Environment e = new Environment();
		for(int i=0; i<1000; i++)
		{
			Clause c = new Clause();
			Literal l = new Literal(i);
			e.add(i, Bool.TRUE);
			f.add(c);
			c.add(l);
		}
		assertEquals(f.eval(e), Bool.TRUE);
	}
	
	@Test
	public void test_all_true_but_one_clauses(){
		Formula f = new Formula();
		Environment e = new Environment();
		for(int i=0; i<1000; i++)
		{
			Clause c = new Clause();
			Literal l = new Literal(i);
			e.add(i, Bool.TRUE);
			if(i==999)
				e.add(i, Bool.FALSE);
			f.add(c);
			c.add(l);
		}
		assertEquals(f.eval(e), Bool.FALSE);
	}	

	@Test
	/**
	 * Tests a formula of 1000 clauses each with 1000 literals
	 * Each clause contains 999 false literals and one true one
	 * So the resulting CNF should be true
	 */
	public void test_1000_by_1000(){
		Formula f = new Formula();
		Environment e = new Environment();
		for(int i=0; i<1000; i++)
		{
			Clause c = new Clause();
			for(int j=0; j<1000; j++)
			{
				Literal l = new Literal(j);
				e.add(j, Bool.FALSE);
				if(j==999)
					e.add(j, Bool.TRUE);
				c.add(l);
			}
			f.add(c);
		}
		assertEquals(f.eval(e), Bool.TRUE);
	}	
	
	@Test
	/**
	 * Tests a formula of 1000 clauses each with 1000 literals
	 * Each clause contains 999 false literals and one undefined one
	 * So the resulting CNF should be true
	 */
	public void test_1000_by_1000_undefined(){
		Formula f = new Formula();
		Environment e = new Environment();
		for(int i=0; i<1000; i++)
		{
			Clause c = new Clause();
			for(int j=0; j<1000; j++)
			{
				Literal l = new Literal(j);
				e.add(j, Bool.FALSE);
				if(j==999)
					e.add(j, Bool.UNDEFINED);
				c.add(l);
			}
			f.add(c);
		}
		assertEquals(f.eval(e), Bool.UNDEFINED);
	}		

	@Test
	/**
	 * Tests a formula of 1000 clauses each with 1000 literals
	 * with a duplicate environment
	 * So the resulting CNF should be true
	 */
	public void test_copied_environment(){
		Formula f = new Formula();
		Environment e = new Environment();
		for(int i=0; i<1000; i++)
		{
			Clause c = new Clause();
			for(int j=0; j<1000; j++)
			{
				Literal l = new Literal(j);
				e.add(j, Bool.FALSE);
				if(j==999)
					e.add(j, Bool.TRUE);
				c.add(l);
			}
			f.add(c);
		}
		Environment e2 = new Environment(e);
		Environment e3 = new Environment();
		Environment e4 = new Environment(e);
		e4.add(999, Bool.FALSE);
		assertEquals(f.eval(e), Bool.TRUE);
		assertEquals(f.eval(e2), Bool.TRUE);
		assertEquals(f.eval(e3), Bool.UNDEFINED);
		assertEquals(f.eval(e4), Bool.FALSE);
	}	
	
	@Test
	public void test_environment_choose(){
		Environment e = new Environment();
		e.add(0, Bool.TRUE);
		e.add(2, Bool.FALSE);
		e.add(102, Bool.UNDEFINED);
		assertEquals(e.choose(), 102);
	}
	
	@Test
	public void test_environment_choose_full(){
		Environment e = new Environment();
		e.add(0, Bool.TRUE);
		e.add(2, Bool.FALSE);
		e.add(102, Bool.TRUE);
		assertEquals(e.choose(), -1);
	}	

}
