package pics;
import photo.Photo;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.ArrayList;
import pics.*;
import javax.swing.tree.*;

public class Album extends PictureElement {

	ArrayList<PictureElement> children;
	String name;
	DefaultMutableTreeNode node;
	
	private static HashMap<String, Album> allAlbums = new HashMap<String, Album>();
	public static Album root = new Album("All Photos");
	
	public Album(String n)
	{
		//If the name is already taken, append _1, _2, etc. after
		int i = 1;
		String newN = n;
		while(allAlbums.get(newN) != null)
		{
			newN = n + "_" + Integer.toString(i);
			i++;
		}
		name = newN;
		children = new ArrayList<PictureElement>();
		allAlbums.put(newN, this);
	}
	
	/**
	 * Creates a new album, or reuses an old one if the name is the same
	 * @param n
	 * @return
	 */
	public static Album make(String n)
	{
		if(allAlbums.get(n) != null)
			return(allAlbums.get(n));
		else
			return(new Album(n));
	}
	
	/**
	 * This will get an album by name, or return null if not found
	 * @param n
	 * @return
	 */
	public static Album getByName(String n)
	{
		return(allAlbums.get(n));
	}
	
	public static Album get(String n)
	{
		return(allAlbums.get(n));
	}
	
	
	//Returns the first parent of the named album
	public static Album getParent(Album child)
	{
		for(Album a : allAlbums.values())
		{
			if(a.children.contains(child))
				return(a);
		}
		return(null);
	}
	
	public void addNode(DefaultMutableTreeNode n)
	{
		node = n;
	}
	
	public DefaultMutableTreeNode getNode(){ return node; }
	
	public DefaultMutableTreeNode addAlbum(Album a)
	{
		children.add(a);
		if(node != null)
		{
			a.node = new DefaultMutableTreeNode(a.name);
			return(a.node);
		}
		return null;
	}
	
	public void addPicture(Picture p)
	{
		children.add(p);
	}
	
	public void addPhotos(Set<Photo> thePhotos)
	{
		for(Photo p : thePhotos)
			this.addPicture(Picture.make(p));
	}
	
	public void removePicture(Picture p)
	{
		if(children.contains(p))
			children.remove(p);
	}
	
	public void removePhotos(Set<Photo> thePhotos)
	{
		for(Photo p : thePhotos)
			this.removePicture(Picture.make(p));
	}
	
	public void removeAlbum(Album a)
	{
		if(children.contains(a))
		{
			children.remove(a);
		}
	}
	
	//This displays this album as the container of all thumbnail contents
	//Differs from include which recursively builds the thumbnail set
	public Set<Photo> display()
	{
		HashSet<Photo> thePhotos = new HashSet<Photo>();
		this.include(thePhotos);
		return(thePhotos);
	}
	
	@Override
	public void include(Set<Photo> thePhotos) {
		for(PictureElement p : children)
		{
			p.include(thePhotos);
		}
	}

}
