package arithmetic;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * JUnit test cases for the Expression visitors
 */
public class ExprVisitorTest {
	
	private Expr sampleExpr; // a sample expression
	
	// for your test cases, you should also create other expressions to demonstrate
	// the correctness of your two visitors
	
	/**
	 * A method showing an example of how an expression may be created
	 * @post sampleExpr represents an expression -0.5*(3 + 19.0)
	 */
	@Before public void createSampleExpr(){
		
		sampleExpr = new Mult(new Const(-0.5), new Plus(new Const(3.0), new Const(19.0)));		
	
	}
	
	@Test
	public void testUnitExpressions(){
		double value;
		EvaluateVisitor eval = new EvaluateVisitor();
		
		Const c = new Const(1.2);
		value = eval.visit(c).doubleValue();
		Assert.assertEquals(value, 1.2, .001);
		
		Mult m = new Mult(new Const(5.0), new Const(4.0));
		value = eval.visit(m).doubleValue();
		Assert.assertEquals(value, 20, .001);
		
		Div d = new Div(new Const(32.0), new Const(0.5));
		value = eval.visit(d).doubleValue();
		Assert.assertEquals(value, 64.0, .001);
		
		Plus p = new Plus(new Const(2.0), new Const(1.0));
		value = eval.visit(p).doubleValue();
		Assert.assertEquals(value, 3.0, .001);
		
		Minus n = new Minus(new Const(15.3), new Const(12.1));
		value = eval.visit(n).doubleValue();
		Assert.assertEquals(value,  3.2, .001);
		
		m = new Mult(new Const(-0.5), new Plus(new Const(3.0), new Const(19.0)));
		value = eval.visit(m).doubleValue();
		Assert.assertEquals(value, -11.0, .001);
	}
	
	@Test
	public void testSignExpressions(){
		Sign value;
		SignVisitor sign = new SignVisitor();
		
		Const c = new Const(1.2);
		value = sign.visit(c);
		Assert.assertEquals(value, Sign.POSITIVE);
		
		Mult m = new Mult(new Const(5.0), new Const(-4.0));
		value = sign.visit(m);
		Assert.assertEquals(value, Sign.NEGATIVE);
		
		Div d = new Div(new Const(32.0), new Const(0.5));
		value = sign.visit(d);
		Assert.assertEquals(value, Sign.POSITIVE);
		
		Plus p = new Plus(new Const(2.0), new Const(-1.0));
		value = sign.visit(p);
		Assert.assertEquals(value, Sign.UNKNOWN);
		
		Minus n = new Minus(new Const(15.3), new Const(12.1));
		value = sign.visit(n);
		Assert.assertEquals(value,  Sign.UNKNOWN);
		
		m = new Mult(new Const(-0.5), new Plus(new Const(3.0), new Const(19.0)));
		value = sign.visit(m);
		Assert.assertEquals(value, Sign.NEGATIVE);
	}	
}
