package words;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import layout.TableLayout;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**
 * WordFinder is an interface for searching a word list.
 * When the user types any part of a word, the interface
 * displays all the words that match.
 */
public class WordFinder extends JFrame {

    private static final long serialVersionUID = 1L;
    
    private final WordList words = new WordList();
    
    private final JTextField find;
    private final JList foundWords;
    private final JScrollPane displayWords;
    private final JButton findButton;
    private final JLabel numWords;
    private final DefaultListModel theWords;
    private final JMenuBar theMenu;
    private final JMenuItem menuFile;
    private final JMenuItem menuExit;
    private final JFileChooser fileChooser;
    
    /**
     * Make a WordFinder window.
     */
    public WordFinder() {
        super("Word Finder");
        
        // call System.exit() when user closes the window
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        Container cp = this.getContentPane();
        /*
         * Task 2: set the layout manager of the content pane to a TableLayout.
         */
        double size[][] = {{5, TableLayout.PREFERRED, 5, TableLayout.FILL, 5, TableLayout.PREFERRED, 5},
        				   {5, TableLayout.PREFERRED, 5, TableLayout.PREFERRED, TableLayout.FILL, 5}};
        this.setLayout(new TableLayout(size));
        
        
        this.add(new JLabel("Find: "), "1, 1");
        this.add(find = new JTextField(20), "3, 1");
        this.add(findButton = new JButton("Search"), "5, 1");
        theWords = new DefaultListModel();
        foundWords = new JList(theWords);
        this.add(displayWords = new JScrollPane(foundWords), "3, 4");
        this.add(numWords = new JLabel("Type to begin search"), "3, 3");
        
        /*
         * Task 1: add an action listener to `find' that outputs matching words
         *         to the console
         */
        find.addActionListener(new ActionListener() {	
        	public void actionPerformed(ActionEvent e)
        	{
        		doSearch();
        	}
        });
        
        /*
         * Task 5: add a Search button
         */
        findButton.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		doSearch();
        	}
        });
        
        /*
         * Task 6: add a File menu with Open... and Exit options
         */
        this.setJMenuBar(theMenu = new JMenuBar());
        JMenu fileMenu;
        theMenu.add(fileMenu = new JMenu("File"));
        fileMenu.add(menuFile = new JMenuItem("Open..."));
        fileMenu.add(menuExit = new JMenuItem("Exit"));
        fileChooser = new JFileChooser();
        
        menuExit.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		System.exit(0);
        	}
        });
        
        menuFile.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		int result = fileChooser.showOpenDialog(theMenu);
        		if(result == JFileChooser.APPROVE_OPTION)
        		{
        			File f = fileChooser.getSelectedFile();
        			try{
        				FileInputStream fIn = new FileInputStream(f);
        				loadWords(fIn);
        			}catch(IOException err){
        				System.err.println(err.getMessage());
        			}
        		}
        	}
        });
        
        this.pack();
        
        try {
            InputStream in = new FileInputStream("words");
            loadWords(in);
        } catch (IOException ioe) { }
    }
    
    private void doSearch()
    {
    	String search = find.getText();
		List<String> found = words.find(search);
		theWords.clear();
		for(String s : found)
			theWords.addElement(s);
		numWords.setText(Integer.toString(found.size())+" matches for '"+search+"'.");
    }
    
    private void loadWords(InputStream in) throws IOException {
        words.load(in);
        // ...
    }
    
    /**
     * Main method.  Makes and displays a WordFinder window.
     * @param args Command-line arguments.  Ignored.
     */
    public static void main(String[] args) {
        // In general, Swing objects should only be accessed from
        // the event-handling thread -- not from the main thread
        // or other threads you create yourself.  SwingUtilities.invokeLater()
        // is a standard idiom for switching to the event-handling thread.
        SwingUtilities.invokeLater(new Runnable() {
            public void run () {
                // Make and display the WordFinder window.
                new WordFinder().setVisible(true);
            }
        });
    }    
}
