package music;

import music.*;
import java.util.ArrayList;

public class Bar implements MusicElement{

	public ArrayList<Elem> elems;
	public int playOn = 0; //0 = play on all repeats, n = play only on nth repetition
	
	public Bar()
	{
		elems = new ArrayList<Elem>();
	}
	
	public Note newNote(Pitch p, double length, Accidental a)
	{
		Note n = new Note(p, length, a);
		elems.add(n);
		return(n);
	}
	
	public Note newNote(Note n)
	{
		elems.add(n);
		return(n);
	}
	
	public Rest newRest(Rest r)
	{
		elems.add(r);
		return(r);
	}
	
	//Chords must be filled in directly
	public Chord newChord()
	{
		Chord c = new Chord();
		elems.add(c);
		return(c);
	}
	
	public Tuplet newTuplet()
	{
		Tuplet t = new Tuplet();
		elems.add(t);
		return(t);
	}
	
	/**
	 * Takes the meter of the given piece and returns true if it
	 * equals the duration of all notes
	 * @param meter
	 * @return
	 */
	public boolean verify(double meter)
	{
		double duration = 0;
		for(Elem e : elems)
		{
			duration += e.duration();
		}
		return(duration==meter);
	}
	
	public void print()
	{
		for(Elem e : elems)
		{
			if(playOn != 0)
				System.err.printf("%d) ", playOn);
			e.print();
			System.err.print(" ");
		}
	}
	
	@Override
	public void accept(MusicVisitor v)
	{
		v.visit(this);
	}
	
}
