package music;

import music.*;
import java.util.HashMap;

public class Key {
	
	public HashMap<Pitch, Accidental> accidentals;
	
	public Key()
	{
		accidentals = new HashMap<Pitch, Accidental>();
	}
	
	public void addAccidental(Pitch pitch, Accidental accidental)
	{
		accidentals.put(pitch, accidental);
	}
	
	public Key(String key)
	{
		accidentals = new HashMap<Pitch, Accidental>();
		
		switch(toKeyCode(key))
		{
		//Keys with added sharps (Note no break statement so C# contains all sharps)
		case 10: case 511: //C# A#m
			accidentals.put(Pitch.make('B'), Accidental.SHARP);
		case 310: case 111: //F# D#m
			accidentals.put(Pitch.make('E'), Accidental.SHARP);
		case 600: case 411: //B G#m
			accidentals.put(Pitch.make('A'), Accidental.SHARP);
		case 200: case 11: //E C#m
			accidentals.put(Pitch.make('D'), Accidental.SHARP);
		case 500: case 311: //A F#m
			accidentals.put(Pitch.make('G'), Accidental.SHARP);
		case 100: case 601: //D Bm
			accidentals.put(Pitch.make('C'), Accidental.SHARP);
		case 400: case 201: //G Em
			accidentals.put(Pitch.make('F'), Accidental.SHARP);
		case 0: case 501: //C Am
			break;
		//keys with added flats
		case 20: case 521: //Cb Abm
			accidentals.put(Pitch.make('F'), Accidental.FLAT);
		case 420: case 221: //Gb Ebm
			accidentals.put(Pitch.make('C'), Accidental.FLAT);
		case 120: case 621: //Db Bbm
			accidentals.put(Pitch.make('G'), Accidental.FLAT);
		case 520: case 301: //Ab Fm
			accidentals.put(Pitch.make('D'), Accidental.FLAT);
		case 220: case 1: //Eb Cm
			accidentals.put(Pitch.make('A'), Accidental.FLAT);
		case 620: case 401: //Bb Gm
			accidentals.put(Pitch.make('E'), Accidental.FLAT);
		case 300: case 101: //F Dm
			accidentals.put(Pitch.make('B'), Accidental.FLAT);
			break;
		}
			
	}
	
	private static int toKeyCode(String key)
	{
		int value = 0;
		switch(key.charAt(0))
		{
		case 'C':
			value = 0; break;
		case 'D':
			value = 100; break;
		case 'E':
			value = 200; break;
		case 'F':
			value = 300; break;
		case 'G':
			value = 400; break;
		case 'A':
			value = 500; break;
		case 'B':
			value = 600; break;
		}
		if(key.length() > 1)
		{
			switch(key.charAt(1))
			{
			case '#':
				value += 10; break;
			case 'b':
				value += 20; break;
			case 'm':
				value += 1; break;
			}
		}
		if(key.length() > 2)
		{
			if(key.charAt(2) == 'm')
				value += 1;
		}
		
		return(value);
	}

}
