package music;

import music.*;
import java.util.HashMap;

public class Pitch {
	
	int tone;
	private static HashMap<Integer, Pitch> allPitches;
	
	public Pitch(char note)
	{
		tone = noteToTone(note);
	}
	
	public int getTone()
	{
		return(tone);
	}
	
	public Pitch(char note, int octaveOffset)
	{
		tone = noteToTone(note) + 7*octaveOffset;
	}
	
	/**
	 * w/ Octave = 0
	 * @param note
	 * @return
	 */
	public static Pitch make(char note)
	{
		return(make(note, 0));
	}
	
	/**
	 * Factory Design Pattern, creates or reuses a Pitch
	 * @param note
	 * @param octaveOffset
	 * @return
	 */
	public static Pitch make(char note, int octaveOffset)
	{
		Integer tone = new Integer(noteToTone(note)+7*octaveOffset);
		if(allPitches == null)
			allPitches = new HashMap<Integer, Pitch>();
		Pitch pitch = allPitches.get(tone);
		if(pitch == null)
		{
			pitch = new Pitch(note, octaveOffset);
		}
		return(pitch);
	}

	
	/**
	 * Returns the integer value for a note (all invalid characters are considered C)
	 * @param note
	 * @return
	 */
	private static int noteToTone(char note)
	{
		switch(note)
		{
		case 'C': return(0);
		case 'D': return(1);
		case 'E': return(2);
		case 'F': return(3);
		case 'G': return(4);
		case 'A': return(5);
		case 'B': return(6);
		case 'c': return(7);
		case 'd': return(8);
		case 'e': return(9);
		case 'f': return(10);
		case 'g': return(11);
		case 'a': return(12);
		case 'b': return(13);
		}
		return(0);
	}
	
	/**
	 * Returns the letter of the pitch plus the number of semitones offset
	 * as a result of an octave shift
	 * @return
	 */
	public int[] toneToNote()
	{
		int[] values = {(int)'C', 0};
		
		switch(tone%7)
		{
		case 0: values[0] = (int)'C'; break;
		case 1: values[0] = (int)'D'; break;
		case 2: values[0] = (int)'E'; break;
		case 3: values[0] = (int)'F'; break;
		case 4: values[0] = (int)'G'; break;
		case 5: values[0] = (int)'A'; break;
		case 6: values[0] = (int)'B'; break;
		}
		
		values[1] = tone/7 * 12;
		
		return(values);
	}
}
