package client;

import javax.swing.JFrame;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import controller.Protocol;
import controller.Tuple;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.PrintWriter;
import java.util.List;

import layout.TableLayout;
import model.Message;

public class ChatroomPane extends JFrame {
	
	private final JLabel displayRoom;
	
	private final DefaultListModel listTranscriptModel;
	private final JList listTranscript;
	private final JScrollPane paneTranscript;
	
	private final JButton reply;
	private final JButton leave;
	
	private final JTextField message;
	
	//------------------------
	
	List<String> transcript;
	String roomname;
	int numGuests;
	PrintWriter writer;
	
	//------------------------
	
	public ChatroomPane(String theRoomname, PrintWriter theWriter)
	{
		super("Chatroom: "+theRoomname);
		roomname = theRoomname;
		numGuests = 0;
		writer = theWriter;
		
		double size[][] = {{5, TableLayout.FILL, 5, TableLayout.PREFERRED, 5},
						   {5, TableLayout.PREFERRED, 5, TableLayout.FILL, 5, TableLayout.PREFERRED, 5}};
		this.setLayout(new TableLayout(size));
		
		//Add the elements
		this.add(displayRoom = new JLabel("Chatroom: "+roomname+" ("+Integer.toString(numGuests)+")"), "1, 1, 1, 1");
		
		this.add(reply = new JButton("Reply"), "3, 5, 1, 1");
		this.add(leave = new JButton("Leave"), "3, 1, 1, 1");
		
		listTranscriptModel = new DefaultListModel();
		listTranscript = new JList(listTranscriptModel);
		this.add(paneTranscript = new JScrollPane(listTranscript), "1, 3, 3, 1");
		
		this.add(message = new JTextField(25), "1, 5, 1, 1");
		
		this.pack();
		
		//Add actions
		reply.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		doReply();
        	}
        });
		message.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		doReply();
        	}
        });	
		leave.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e)
        	{
        		doLeave();
        	}
        });	

		this.addWindowListener(
			    new WindowAdapter() {
			        public void windowClosing(WindowEvent e) {
			            doLeave();
			        }
			    }
			);
		
	}
	
	//-----------------ACTIONS----------------------
	public void doReply(){
		String msg = message.getText();
		msg = Message.makeSafe(msg);
		message.setText("");
		writer.println("[MSG]["+roomname+"]["+msg+"]");
		writer.flush();
	}
	
	public void doLeave(){
		writer.println("[LEAVE]["+roomname+"]");
		writer.flush();
		this.setVisible(false);
	}
	
	
	//----------------------------------------------
	
	/**
	 * Updated when there is a change in the transcript or numGuests
	 */
	public void update(Protocol update)
	{
		switch(update.command){
		//Change the transcript pane
		case TRANSCRIPT:
			listTranscriptModel.clear();
			String[] messages = update.textField.split("`");
			listTranscriptModel.ensureCapacity(messages.length);
			for(int i=0; i<messages.length; i++)
			{
				listTranscriptModel.addElement(messages[i]);
			}
			listTranscript.ensureIndexIsVisible(messages.length-1);
			break;
		case ROOMLIST:
			numGuests = 0;
			for(Tuple t : update.roomList)
				if(t.name.equals(roomname))
					numGuests = t.number;
			displayRoom.setText("Chatroom: "+roomname+" ("+Integer.toString(numGuests)+")");
			break;
		default:
			System.err.println("Warning, attempted to update chatroom with protocol type: "+update.command.toString());
			break;
		}
	}

	//Testing method to view interface (commands will fail, but useful for display)
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
            public void run () {
                // Make and display the Chatroom window.
                new ChatroomPane("Room", null).setVisible(true);
            }
        });		
	}
}
