package controller;

import java.util.List;
import java.util.ArrayList;
import java.util.regex.*;

public class Protocol {
	
	public Command command;
	public String nameField;
	public String textField;
	public List<String> userList;
	public List<Tuple> roomList;
	public ClientHandler source;
	
	/**
	 * This command will parse a protocol from a given line of text, if the protocol is invalid
	 * will throw a warning.
	 * @param text
	 */
	public Protocol(String text)
	{
		command = Command.NONE;
		nameField = "";
		textField = "";
		userList = new ArrayList<String>();
		roomList = new ArrayList<Tuple>();
		
		//First tokenize the string on '[' and ']'
		String[] rawTokens = text.split("\\[|\\]");
		List<String> tokens = new ArrayList<String>();
		//Remove all 0-length tokens and make iterable
		for(int i=0; i<rawTokens.length; i++)
			if(rawTokens[i].length() > 0)
				tokens.add(rawTokens[i]);
		
		for(String token : tokens)
		{
			//Parse the COMMAND header
			if(command == Command.NONE)
			{
				if(token.equals("CREATE"))
					command = Command.CREATE;
				else if(token.equals("JOIN"))
					command = Command.JOIN;
				else if(token.equals("ALIAS"))
					command = Command.ALIAS;
				else if(token.equals("LEAVE"))
					command = Command.LEAVE;
				else if(token.equals("MSG"))
					command = Command.MSG;
				else if(token.equals("INVITED"))
					command = Command.INVITED;
				else if(token.equals("ASSIGNED"))
					command = Command.ASSIGNED;
				else if(token.equals("TRANSCRIPT"))
					command = Command.TRANSCRIPT;
				else if(token.equals("ROOMLIST"))
					command = Command.ROOMLIST;
				else if(token.equals("USERLIST"))
					command = Command.USERLIST;
				else
					System.err.println("Warning, ["+token+"] is an unsupported command.");
			}
			else if((command == Command.CREATE || command == Command.JOIN || command == Command.ALIAS ||
					command == Command.INVITED || command == Command.ASSIGNED || command == Command.LEAVE ||
					command == Command.MSG || command == Command.TRANSCRIPT) && nameField.length() == 0)
			{
				nameField = token;
			}
			else if((command == Command.TRANSCRIPT || command == Command.MSG) && nameField.length()>0)
			{
				textField = token;
			}
			else if(command == Command.USERLIST)
			{
				userList.add(token);
			}
			else if(command == Command.ROOMLIST)
			{
				roomList.add(new Tuple(token));
			}
		}
		
	}
	
	/**
	 * Alternate protocols for server-generated requests on the model
	 */
	public Protocol(Command action, String data)
	{
		nameField = data;
		command = action;
	}
	
	public Protocol(Command action, List<Tuple> rooms)
	{
		command = action;
		roomList = rooms;
	}
	
	public Protocol(List<String> users)
	{
		command = Command.USERLIST;
		userList = users;
	}
	
	public Protocol(Command action, String name, String text)
	{
		command = action;
		nameField = name;
		textField = text;
	}

}
