package main;

import javax.swing.SwingUtilities;

import controller.Command;
import controller.Protocol;

import client.HubPane;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;

import main.Server;

/**
 * GUI chat client runner.
 */
public class Client{

    
    public static final String HOSTNAME = "localhost";
    
    PrintWriter writer;
    BufferedReader reader;
    Socket clientSocket;
    
    HubPane theHub;
    
    
    public static void main(String[] args) {
    	 Client GUIClient = new Client();
    	 GUIClient.run();
    }

	
	public void run() {
		try {
            clientSocket = new Socket(HOSTNAME, Server.PORT);
            writer = new PrintWriter(clientSocket.getOutputStream(), true);
            reader = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
    		SwingUtilities.invokeLater(new Runnable() {
                public void run () {
                    // Make and display the HubPane window.
                    theHub = new HubPane(writer);
                    theHub.setVisible(true);
                }
            });  
    		//Run the client communication (listen for server updates)
    		try{
    			String line = reader.readLine();
    			while(line != null)
    			{
    				line = line.trim();
    				if(line.length()>0 && theHub != null)
    				{
    					Protocol command = new Protocol(line);
    					theHub.update(command);
    					line = reader.readLine();
    				}
    			}
    			reader.close();
    			writer.close();
    			System.out.println("Closing client...");
    		}catch(IOException e){
    			System.err.println("Error while trying to listen to server"+e.getMessage());
    		}    		
    		
        } catch (UnknownHostException e) {
            System.err.println("Couldn't locate the host "+HOSTNAME+".");
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Sorry, server is currently offline!");
            System.exit(1);
        }
	}
}
